@@ -51,7 +51,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype ToolTip
     \inherits Popup
-    \instantiates QQuickToolTip
+//!     \instantiates QQuickToolTip
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-popups
@@ -124,20 +124,14 @@ class QQuickToolTipPrivate : public QQuickPopupPrivate
     Q_DECLARE_PUBLIC(QQuickToolTip)
 
 public:
-    QQuickToolTipPrivate()
-        : delay(0),
-          timeout(-1)
-    {
-    }
-
     void startDelay();
     void stopDelay();
 
     void startTimeout();
     void stopTimeout();
 
-    int delay;
-    int timeout;
+    int delay = 0;
+    int timeout = -1;
     QString text;
     QBasicTimer delayTimer;
     QBasicTimer timeoutTimer;
@@ -194,7 +188,7 @@ void QQuickToolTip::setText(const QString &text)
         return;
 
     d->text = text;
-    setAccessibleName(text);
+    maybeSetAccessibleName(text);
     emit textChanged();
 }
 
@@ -244,12 +238,13 @@ void QQuickToolTip::setTimeout(int timeout)
     if (d->timeout == timeout)
         return;
 
+    d->timeout = timeout;
+
     if (timeout <= 0)
         d->stopTimeout();
     else if (isVisible())
         d->startTimeout();
 
-    d->timeout = timeout;
     emit timeoutChanged();
 }
 
@@ -257,9 +252,18 @@ void QQuickToolTip::setVisible(bool visible)
 {
     Q_D(QQuickToolTip);
     if (visible) {
-        if (!d->visible && d->delay > 0) {
-            d->startDelay();
-            return;
+        if (!d->visible) {
+            // We are being made visible, and we weren't before.
+            if (d->delay > 0) {
+                d->startDelay();
+                return;
+            }
+        } else {
+            // We are being made visible, even though we already were.
+            // We've probably been re-opened before our exit transition could finish.
+            // In that case, we need to manually start the timeout, as that is usually
+            // done in itemChange(), which won't be called in this situation.
+            d->startTimeout();
         }
     } else {
         d->stopDelay();
@@ -276,9 +280,40 @@ QQuickToolTipAttached *QQuickToolTip::qmlAttachedProperties(QObject *object)
     return new QQuickToolTipAttached(object);
 }
 
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlmethod void QtQuick.Controls::ToolTip::show(string text, int timeout)
+
+    This method shows the \a text as a tooltip, which times out in
+    \a timeout (milliseconds).
+*/
+void QQuickToolTip::show(const QString &text, int ms)
+{
+    if (ms >= 0)
+        setTimeout(ms);
+    setText(text);
+    open();
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlmethod void QtQuick.Controls::ToolTip::hide()
+
+    This method hides the tooltip.
+*/
+void QQuickToolTip::hide()
+{
+    close();
+}
+
 QFont QQuickToolTip::defaultFont() const
 {
-    return QQuickControlPrivate::themeFont(QPlatformTheme::TipLabelFont);
+    return QQuickTheme::font(QQuickTheme::ToolTip);
+}
+
+QPalette QQuickToolTip::defaultPalette() const
+{
+    return QQuickTheme::palette(QQuickTheme::ToolTip);
 }
 
 void QQuickToolTip::itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &data)
@@ -303,10 +338,14 @@ void QQuickToolTip::timerEvent(QTimerEvent *event)
     if (event->timerId() == d->timeoutTimer.timerId()) {
         d->stopTimeout();
         QQuickPopup::setVisible(false);
-    } else if (event->timerId() == d->delayTimer.timerId()) {
+        return;
+    }
+    if (event->timerId() == d->delayTimer.timerId()) {
         d->stopDelay();
         QQuickPopup::setVisible(true);
+        return;
     }
+    QQuickPopup::timerEvent(event);
 }
 
 #if QT_CONFIG(accessibility)
@@ -321,7 +360,7 @@ void QQuickToolTip::accessibilityActiveChanged(bool active)
     QQuickPopup::accessibilityActiveChanged(active);
 
     if (active)
-        setAccessibleName(d->text);
+        maybeSetAccessibleName(d->text);
 }
 #endif
 
@@ -330,16 +369,10 @@ class QQuickToolTipAttachedPrivate : public QObjectPrivate
     Q_DECLARE_PUBLIC(QQuickToolTipAttached)
 
 public:
-    QQuickToolTipAttachedPrivate()
-        : delay(0),
-          timeout(-1)
-    {
-    }
-
     QQuickToolTip *instance(bool create) const;
 
-    int delay;
-    int timeout;
+    int delay = 0;
+    int timeout = -1;
     QString text;
 };
 
@@ -355,7 +388,7 @@ QQuickToolTip *QQuickToolTipAttachedPrivate::instance(bool create) const
     if (!tip && create) {
         // TODO: a cleaner way to create the instance? QQml(Meta)Type?
         QQmlComponent component(engine);
-        component.setData("import QtQuick.Controls 2.2; ToolTip { }", QUrl());
+        component.setData("import QtQuick.Controls 2.4; ToolTip { }", QUrl());
 
         QObject *object = component.create();
         if (object)
@@ -515,10 +548,9 @@ void QQuickToolTipAttached::show(const QString &text, int ms)
     tip->resetWidth();
     tip->resetHeight();
     tip->setParentItem(qobject_cast<QQuickItem *>(parent()));
-    tip->setTimeout(ms >= 0 ? ms : d->timeout);
     tip->setDelay(d->delay);
-    tip->setText(text);
-    tip->open();
+    tip->setTimeout(ms >= 0 ? ms : d->timeout);
+    tip->show(text);
 }
 
 /*!
@@ -534,8 +566,9 @@ void QQuickToolTipAttached::hide()
     QQuickToolTip *tip = d->instance(false);
     if (!tip)
         return;
-
-    tip->close();
+    // check the parent item to prevent unexpectedly closing tooltip by new created invisible tooltip
+    if (parent() == tip->parentItem())
+        tip->close();
 }
 
 QT_END_NAMESPACE
